// DFS 1101 SPI Arduino control V 3.1 by OK1DXD, (c) 2016, 2017
//-------------------------------------------------------------
// Temperature sensor Dallas ds18b20
// Optional Humidity/Temperature sensor DHT11
// BME280/BMP280 outside temp/presure/humidity sensor with no calibration
// Beacon ON/OFF controll by PLL lock check possible
// Mode pin to choose beep or cw beacon mode
// GENERATOR pin to choose RF Generator mode (a loong key down)
// Direct SIEMENS PMB2306 PLL control
// This program loads the PLL with precalculated status and divider data
// LSB appears as the first bit
// AF4JF Morse coder included
// 
#include <Morse2.h>   // a morse code library modified by AF4JF
//#include <dht.h>    // humidity
#include <OneWire.h>
#include <DallasTemperature.h> // temperature sensor
#include <Wire.h>
#include <SPI.h>
#include <Adafruit_Sensor.h>

//choose the right library here
//#include <Adafruit_BME280.h> // I2C on 0x76 modification needed
//Adafruit_BME280 bme; // I2C on 0x76

#include <Adafruit_BMP280.h> //choose the another one
Adafruit_BMP280 bme; // I2C on 0x76 modification needed

// Beacon setup
const int mode = 0;         // Mode 0= Beacon / 1= Foxhunt (tone on beep pin)
const int key_speed = 15;   // CW send speed
const int beep_pin = 9;     // Pin for CW tone, !!! MUST be a PWM pin !!!
int key_pin1 = 13;          // Pin for positive PTT/Rig Key
int key_pin2 = 3;           // Pin for negative complementary PTT/Rig Key
int mode_pin = 12;          // Pin for mode set on startup  /tone mode if pressed (LOW) on powerUp
int generator = 8;          // Pin for RF Generator mode (a loooong key down if LOW on powerUP)
int frq1 = 10;              // Pin for frequency setup (jumper1)
int frq2 = 11;              // Pin for frequency setup (jumper2)

Morse2 morse2(beep_pin, key_pin1, key_speed, 1); // Set up the Morse Library for use

int voltage_in = 0;        // Analog pin input for mains voltage
float v_in = 0 ;           // voltage initial value

// Temperature measurement setup
int temp_pin = 2; // pin for DS18b20 temperature sensors, must be pulled up by external resistor
OneWire oneWire(temp_pin);
// declare as sensor reference by passing oneWire reference to Dallas Temperature.
DallasTemperature sensors(&oneWire);
// SENSOR ADDRESS ID:
DeviceAddress tempSensor01 = {0x28, 0xFF, 0xB9, 0xFB, 0xB3, 0x16, 0x05, 0x30};

const char temp_text[3] = "  ";  //outside temp
const char temp_text2[3] = "  "; //inside temp
const char v_text[3] = "   ";    //voltage
const char h_text[3] = " ";      //humidity
const char p_text[4] = " ";      //pressure

// define PLL Lock LED, Lock input pin
int lock_LED = 13;     // pin PLL Lock LED
int pll_Lock_IN = 4;   // pin PLL LOCK signal from DFS1101

//  SPI PLL Control setup
int CLK = 6;
int SDI = 7;
int CS =  5;

int Lock_OK = 0;  // variable to store the readed PLL_LOCK status
int ii = 10  ;     // number of sec ticks when beacon OFF

void setup()
{
  // Beacon initial setup
  pinMode(key_pin1, OUTPUT);      // set PTT/Key pin to output
  pinMode(key_pin2, OUTPUT);      // set PTT/Key pin to output  COMPLEMENTARY SIGNAL FOR PIN SWITCH

  // Jumper for RF Generator mode
  pinMode(generator, INPUT);      // set mode pin to input
  digitalWrite(generator, HIGH);  // pull up resistor on button for mode switch on power on
  //
  // jumpers for frequency setup
  pinMode(frq1, INPUT);           // set mode pin to input
  digitalWrite(frq1, HIGH);       // pull up resistor on button for frq setup on power on
  pinMode(frq2, INPUT);           // set mode pin to input
  digitalWrite(frq2, HIGH);       // pull up resistor on button for frq setup on power on

  // This is the waiting status at the begin of the start sequence .. no signal should be transmitted
  digitalWrite(key_pin1, LOW); // Turn off the PTT/Key relay
  digitalWrite(key_pin2, HIGH); // Turn off the PTT/Key relay

  // Set the Key mode and LED for Beacon or Foxhunt (beep) by Mode_JUMPER
  pinMode(mode_pin, INPUT);           // set mode pin to input
  digitalWrite(mode_pin, HIGH);       // pull up resistor on button for mode switch on power up
  if (digitalRead(mode_pin) == HIGH)  // Set the Key mode and LED for Beacon if button not pressed
  { // Set Beacon Mode
    Morse2(key_pin1, key_pin2, key_speed, 0);  // Set up to key the Beacon
  } else { // Set Beep Mode
    Morse2(beep_pin, key_pin2, key_speed, 1);  // Set up to send audio beeps only
  }

  // set these SPI pins as an output:
  pinMode( CS, OUTPUT);
  pinMode(CLK, OUTPUT);
  pinMode(SDI, OUTPUT);

  // get the CS pin high to de-select the chip:
  digitalWrite(CS, HIGH);
  digitalWrite(CLK, LOW);

  //PLL LOCK LED
  pinMode(lock_LED, OUTPUT);        // set the LED pin as output
  pinMode(pll_Lock_IN, INPUT);      // set the Lock pin as input
  digitalWrite(pll_Lock_IN, HIGH);  // set the digital pin pull up resistor

  Serial.begin(9600);  // initialize serial monitor to 9600Bd
  delay(1000);         //  initial delay after the power on

  // BME280 begin
  bme.begin();

  // Start up the DallasTemperature library
  sensors.begin();
  // Set Measurement Resolution
  sensors.setResolution(tempSensor01, 10); //10bit is good enough

  // PLL setup
  // status and R, A/N counter telegrams calculated externaly (by OK1DX MS Exceltable)

  //16 bit Status (PMB2306 Dual Mode)msg, same for all FRQ, appended by 16 zeroes
  uint32_t status2 = 0b11111100010011010000000000000000;     // 0xFC4D0000
  // uint32_t status1 = 0b10011100010011010000000000000000; // test mode = PLL OFF

  /* for the 92,5MHz internal reference followin constants used:
    f0 = 10368,000     R=185 A=2 N=47
    f1 = 10800,000     R=37  A=2 N=20
    f2 = 10803,000     R=185 A=5 N=101
    f3 = 11000,000     R=37  A=2 N=25
    f4 = 10368,880 R=1000  A=0 N=256 //beacon FRQ
  */

  // choose the FRQ to be set
   int pointer = 4; // set the the default frequency pointer, index starts at 0 !

  if ((digitalRead(frq1) == HIGH) && (digitalRead(frq2) == HIGH)) pointer=4 ;
  if ((digitalRead(frq1) == HIGH) && (digitalRead(frq2) == LOW))  pointer=3 ;
  if ((digitalRead(frq1) == LOW)  && (digitalRead(frq2) == HIGH)) pointer=2 ;
  if ((digitalRead(frq1) == LOW)  && (digitalRead(frq2) == LOW))  pointer=1 ;
  
  int freq[5] = {10368, 10800, 10803, 11000, 10368}; //the last is the Beacon FRQ

  //18 bit msg, appended by 14 zeroes to 32 bits
  uint32_t rcounter[5] =  {0x00B9C000, 0x0025C000, 0x00B9C000, 0x0025C000, 0x03E8C000}; //
  //23 bit msg, appended by 9 zeroes to 32 bits
  uint32_t ancounter[5] = {0x04017C00, 0x0400A400, 0x0A032C00, 0x0400CC00, 0x00080400}; ///must finish with bin adress 10

  Serial.println("** DFS1101 direct PMB2306 PLL control **");
  Serial.println("-------------------------------");
  Serial.print("FRQ Pointer:  ");
  Serial.println(pointer);
  Serial.print("FRQ   [MHz]:  ");
  Serial.println(freq[pointer]);
  Serial.print("FRQ/2 [MHz]:  ");
  Serial.println(freq[pointer] / 2);
  Serial.print("FRQ/8 [MHz]:  ");
  Serial.println(freq[pointer] / 8); // for an external prescaler reading
  Serial.print("VCO   [MHz]:  ");
  Serial.println(freq[pointer] / 4);
  Serial.print("Status:      ");
  Serial.println(status2, HEX);
  Serial.print("R counter:   ");
  Serial.println(rcounter[pointer], HEX);
  Serial.print("N/A counter: ");
  Serial.println(ancounter[pointer], HEX);
  Serial.println("-------------------------------");

  //  Now going to send these all registers via SPI:
  Serial.println("* SPI MSB TRANSFER STARTED ->");
  Serial.println();
  // status2
  digitalWrite(CS, LOW);
  delay(10);
  shiftOut32(SDI, CLK, status2, 16 ); // 16bit status
  delay(10);
  digitalWrite(CS, HIGH);
  Serial.println("* Status SENT *");

  // R counter
  digitalWrite(CS, LOW);
  delay(10);
  shiftOut32(SDI, CLK, rcounter[pointer], 18 ); // 18 bit R counter
  delay(10);
  digitalWrite(CS, HIGH);
  Serial.println("* R-counter SENT *");

  // N/A counter
  digitalWrite(CS, LOW);
  delay(10);
  shiftOut32(SDI, CLK, ancounter[pointer], 23 ); // 23 bit N/A counter
  delay(10);
  digitalWrite(CS, HIGH);
  Serial.println("* AN-counter SENT *");

  // when all data sent
  Serial.println();
  Serial.println("* PLL SETUP FINISHED *");
  Serial.println();
  delay(1000); // delay to settle a little bit
}

// sent serial bits (of lenght) telegram to SPI, dumps bin value on Serial.Monitor if needed
void shiftOut32(uint8_t dataPin, uint8_t clockPin, uint32_t val, uint8_t lenght)
{
  uint8_t i;
  unsigned long temp = 0;
  /*    Serial.print("Lenght: ");
      Serial.println(lenght);
      Serial.print("val[BIN]: ");
      Serial.println(val, BIN);
  */

  //loop sends MSB first
  for (i = 0; i < lenght; i++)  {
    temp = val & 0b10000000000000000000000000000000; //mask a MSB bit
    /*
            // dumps on Serial Monitor if need to check
            if (temp == 0) Serial.print("0");
            else Serial.print("1");
    */
    // send data bit
    if (temp == 0) digitalWrite(dataPin, LOW);
    else digitalWrite(dataPin, HIGH);

    val = val << 1;    // shift a one bit left

    // pulse a Clock
    delay (1);
    digitalWrite(clockPin, HIGH);
    delay (1);
    digitalWrite(clockPin, LOW);
    delay (1);
  }
}

void loop()
{
  // If PLL LOCK, then BEACON ON
  //  if (digitalRead (pll_Lock_IN) == HIGH)
  //  {
 
 // Test on RF generator mode first :
  if (digitalRead(generator) == HIGH) { // Run as RF generator if jumper open
    Serial.println("RF Generator ON");
    digitalWrite(lock_LED, HIGH); // LED ON
    digitalWrite(key_pin2, LOW); // Key Down
    delay(240000); // generator key down time (240s=4min)
    int key_speed = 35; // a fast beacon text will be send then
    digitalWrite(key_pin2, HIGH); // Key Up
    digitalWrite(lock_LED, LOW); // LED OFF
    Serial.println("RF Generator OFF");
    }

  Serial.print("[");
  Serial.print(millis() / 1000);
  Serial.print("s] *Beacon ON* ");
  
  // IF Beacon mode then initial key down
  if (digitalRead(mode_pin) == HIGH) {
    digitalWrite(lock_LED, HIGH); // LED ON
    digitalWrite(key_pin2, LOW); // Key Down
    delay(10000); // initial key down time
    digitalWrite(key_pin2, HIGH); // Key Up
    digitalWrite(lock_LED, LOW); // LED OFF
    }
    
  //  sent basic Beacon text and weather readings
  morse2.sendmsg(" VVV de OK1DXD Beacon LOC JN79IX "); // sent Beacon text
  
  // get enclosure temperature
  sensors.requestTemperaturesByAddress(tempSensor01);
  //Inside Temperature first from DS18B20
  Serial.print("Int_TMP:");
  Serial.print(sensors.getTempC(tempSensor01)); //inside temperature
  Serial.print("*C  ");
  //CW
  morse2.sendmsg(" Int TMP "); // sent Beacon tmp
  dtostrf ((sensors.getTempC(tempSensor01)), 3, 0, temp_text); //dtostrf(floatVar, minStringWidthIncDecimalPoint, numVarsAfterDecimal, charBuf);
  morse2.sendmsg(temp_text);
  morse2.sendmsg("C ");

  //BMP280 Outside Temperature
  Serial.print("Out_TMP:");
  Serial.print(bme.readTemperature()); 
  Serial.print("*C ");
  //CW
  morse2.sendmsg("OUT TMP "); // sent Beacon text
  dtostrf (bme.readTemperature(), 3, 0, temp_text2);
  morse2.sendmsg(temp_text2);
  morse2.sendmsg("C ");

  //BMP280 Presure
  Serial.print("Pressure:");
  Serial.print(bme.readPressure() / 94.5); //simply correction put here
  Serial.print("hPa ");
  //CW
  morse2.sendmsg("PRESS ");
  dtostrf ((bme.readPressure() / 94.5), 4, 0, p_text); 
  morse2.sendmsg(p_text);
  morse2.sendmsg("hPa ");
/*
  //only if BME280 - Humidity sensor used
  Serial.print("Humidity:");
  Serial.print(bme.readHumidity());
  Serial.print("% ");
  //CW
  morse2.sendmsg("HUM ");
  dtostrf ((bme.readHumidity()), 4, 0, p_text);
  morse2.sendmsg(p_text);
  morse2.sendmsg(" ");
*/
  // Altitude only to be sent on the SerialPrint
  Serial.print("Alt:");
  Serial.print(bme.readAltitude(1030.00)); // this should be adjusted to your local forcase
  Serial.print("m ");

  // Mains Voltage at the last
  int sensorValue = analogRead(A7);  // get value on analog pin
  float voltage = sensorValue * (5.03 / 1023.0); // Convert analog reading to a voltage (0 - 5V):
  float divider = (11.79 / 14.2) * (22 + 5.6) / 5.6; // Calibration rate (actual maesured/sensor value)* 22K/5K6 voltage divider, would be precise calibrated here
  Serial.print("Uin:");
  Serial.print(divider * voltage, 1);
  Serial.print("V");
  //CW
  morse2.sendmsg(" Uin ");
  dtostrf ((divider * voltage), 3, 1, v_text);
  morse2.sendmsg(v_text);
  morse2.sendmsg("V   de OK1DXD/B +K");
  Serial.println(" *BEACON OFF*");

  // beacon loop delay
  for (ii = 0; ii < 60; ii++) // nr of sec ticks before next turn
  {
    delay(990);
    digitalWrite(beep_pin, HIGH);
    delay (10);
    digitalWrite(beep_pin, LOW);
  //  delay (1)
  }  
  //
  //  }
  /*   else {
     digitalWrite(lock_LED, LOW);  // LED off
     digitalWrite(key_pin2, HIGH); // Key Up
     Serial.println("No PLL lock -> Beacon OFF");
     delay(300);
     }
  */
}

